/*!
*
* RaphaëlTools : Raphaël extension (button, sliders, misc..)
*                    version 0.5.1 (2011-10-06)
*
*                 Copyright (C)  2011  Jan Stransky
*
*      Czech Technical University, Faculty of Civil Engineering,
*  Department of Structural Mechanics, 166 29 Prague, Czech Republic
*
*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @fileOverview <a href="http://raphaeljs.com/">Raphaël</a> (svg javascript library) extension by various functions and "classes" (slider, button, etc.). Requires <a href="http://raphaeljs.com/">raphael.js</a> loaded before loading this file. For more information see <a href="http://mech.fsv.cvut.cz/~stransky/software/raphaeltools/">project homepage</a>.
<br /><br/>RaphaëlTools is a free software distributed under <a href='http://www.gnu.org/licenses/gpl.html'>GNU GPL license</a>.
* @author <a href="http://mech.fsv.cvut.cz/~stransky/">Jan Stránský</a>
* @version Version 0.5.1 (2011-10-06)
*/

/** Generic function used for draging ojects. For each dragable object should be defined function dragUpdate(dx,dz): e.g. moving only in x direction etc.
* @param {float} dx x increment of dragging
* @param {float} dz z increment of dragging
* @example
* a = new Arrow(100,59,199,30) // equivalent to a = Arrow.create(100,59,199,30);
* a.dragUpdate = function(dx,dz) { a.translate(dx,dz); }
* a.drag(move,init,reset) // make an object draggable using a.dragUpdate() function when moving*/
function dragMove(dx,dz) {
	this.dragUpdate(dx - (this.deltax || 0), dz - (this.deltaz || 0));
	this.deltax = dx;
	this.deltaz = dz;
}

/** Generic function used for draging ojects, see {@link dragMove}*/
function dragInit() { this.deltax = this.deltaz = 0; }

/** Generic function used for draging ojects, see {@link dragMove}*/
function dragReset() { this.dragReset(); }


/* ***********************************************************
* other generic functions
************************************************************/
/** Raphael function loaded from <a href="http://raphaeljs.com">raphael.js file</a>
* @class*/
Raphael = Raphael;

/** Raphael.el element of {@link Raphael}
* @memberOf Raphael#
*/
Raphael.el = Raphael.el;

/**#@+
* @function
* @memberOf Raphael#
*/
// color setting
/** Sets fill color of receiver to red*/     Raphael.el.fillRed     = function() { this.attr({"fill":"#f00"}); }
/** Sets fill color of receiver to green*/   Raphael.el.fillGreen   = function() { this.attr({"fill":"#0f0"}); }
/** Sets fill color of receiver to blue*/    Raphael.el.fillBlue    = function() { this.attr({"fill":"#00f"}); }
/** Sets fill color of receiver to yellow*/  Raphael.el.fillYellow  = function() { this.attr({"fill":"#ff0"}); }
/** Sets fill color of receiver to cyan*/    Raphael.el.fillCyan    = function() { this.attr({"fill":"#0ff"}); }
/** Sets fill color of receiver to magenta*/ Raphael.el.fillMagenta = function() { this.attr({"fill":"#f0f"}); }
/** Sets fill color of receiver to orange*/  Raphael.el.fillOrange  = function() { this.attr({"fill":"#f50"}); }
/** Sets fill color of receiver to black*/   Raphael.el.fillBlack   = function() { this.attr({"fill":"#000"}); }
/** Sets fill color of receiver to white*/   Raphael.el.fillWhite   = function() { this.attr({"fill":"#fff"}); }
/** Sets fill color of receiver to gray*/    Raphael.el.fillGray    = function() { this.attr({"fill":"#777"}); }
/** Sets stroke color of receiver to red*/     Raphael.el.strokeRed     = function() { this.attr({"stroke":"#f00"}); }
/** Sets stroke color of receiver to green*/   Raphael.el.strokeGreen   = function() { this.attr({"stroke":"#0f0"}); }
/** Sets stroke color of receiver to blue*/    Raphael.el.strokeBlue    = function() { this.attr({"stroke":"#00f"}); }
/** Sets stroke color of receiver to yellow*/  Raphael.el.strokeYellow  = function() { this.attr({"stroke":"#ff0"}); }
/** Sets stroke color of receiver to cyan*/    Raphael.el.strokeCyan    = function() { this.attr({"stroke":"#0ff"}); }
/** Sets stroke color of receiver to magenta*/ Raphael.el.strokeMagenta = function() { this.attr({"stroke":"#f0f"}); }
/** Sets stroke color of receiver to orange*/  Raphael.el.strokeOrange  = function() { this.attr({"stroke":"#f50"}); }
/** Sets stroke color of receiver to black*/   Raphael.el.strokeBlack   = function() { this.attr({"stroke":"#000"}); }
/** Sets stroke color of receiver to white*/   Raphael.el.strokeWhite   = function() { this.attr({"stroke":"#fff"}); }
/** Sets stroke color of receiver to gray*/    Raphael.el.strokeGray    = function() { this.attr({"stroke":"#777"}); }
/** Sets fill and stroke color of receiver to red*/     Raphael.el.red     = function() { this.fillRed();     this.strokeRed(); }
/** Sets fill and stroke color of receiver to green*/   Raphael.el.green   = function() { this.fillGreen();   this.strokeGreen(); }
/** Sets fill and stroke color of receiver to blue*/    Raphael.el.blue    = function() { this.fillBlue();    this.strokeBlue(); }
/** Sets fill and stroke color of receiver to yellow*/  Raphael.el.yellow  = function() { this.fillYellow();  this.strokeYellow(); }
/** Sets fill and stroke color of receiver to cyan*/    Raphael.el.cyan    = function() { this.fillCyan();    this.strokeCyan(); }
/** Sets fill and stroke color of receiver to magenta*/ Raphael.el.magenta = function() { this.fillMagenta(); this.strokeMagenta(); }
/** Sets fill and stroke color of receiver to orange*/  Raphael.el.orange  = function() { this.fillOrange();  this.strokeOrange(); }
/** Sets fill and stroke color of receiver to black*/   Raphael.el.black   = function() { this.fillBlack();   this.strokeBlack(); }
/** Sets fill and stroke color of receiver to white*/   Raphael.el.white   = function() { this.fillWhite();   this.strokeWhite(); }
/** Sets fill and stroke color of receiver to gray*/    Raphael.el.gray    = function() { this.fillGreen();   this.strokeGray(); }
/** Set width of receiver
* @param w new width*/
Raphael.el.setWidth = function(w) { this.attr({"stroke-width":w}); }
/**#@-*/

/** Returns [x,z] components of transformated vector
* @param {float} x x component of original vector
* @param {float} z z component of original vector
* @param {float} sin sin of angle of rotation
* @param {float} cos cos of angle of rotation
* @returns {[float,float]} coordinates of transformated vector*/
vecTrans = function(x,z,sin,cos) { return [x*cos-z*sin,x*sin+z*cos]; }



/** 2d xz plane point implementation
* @class represents points in 2d zx plane
* @param {float} x x coordinate of point
* @param {float} z z coordinate of point
* @property {float} x x coordinate
* @property {float} z z coordinate
* @property {[Points2d]} copies array of copies of itself
* @property {[[float,float]]} copiesShifts horizontal and verical shifts of copies from receiver*/
Point2d = function(x,z) {
	this.x = x || 0;
	this.z = z || 0;
	this.copies = [];
	this.copiesShifts = []

	/** String representation
	* @returns {String} string representation*/
	this.toString = function() {
		return "Point2d ( "+this.x+","+this.z+" )";
	}

	/** Copy receiver to given positions
	* @param {[[float,float]]} shifts vertical and horizontal shifts of copied points
	* @example n = Point2d(100,150);
	* shifts=[[100,200],[200,200],[100,300],[200,300]]*/
	this.copy = function(shifts) {
		for (var i=0; i<shifts.length; i++) {
			this.copiesShifts.push(shifts[i]);
			this.copies.push(new Point2d(this.x+shifts[i][0],this.z+shifts[i][1]));
		}
	}

	/** Sets new position to receiver
	* @param {float} x x coordinate of new position
	* @param {float} z z coordinate of new position*/
	this.setXZ = function(x,z) {
		this.x=x; this.z=z; this.up();
	}

	/** Set new x coordinate to receiver
	* @param x x coordinate of new position*/
	this.setX = function(x) {
		this.x=x; this.up();
	}

	/** Set new z coordinate to receiver
	* @param z z coordinate of new position*/
	this.setZ = function(z) {
		this.z=z; this.up();
	}

	/** Change position of receiver by given values
	* @param {float} dx length of translation in x direction
	* @param {float} dz length of translation in z direction*/
	this.translate = function(dx,dz) {
		this.x+=dx; this.z+=dz; this.up();
	}

	/** Change horizontal position of receiver by given values
	* @param {float} dx length of translation in x direction*/
	this.translateX = function(dx) {
		this.x+=dx; this.up();
	}

	/** Change vertical position of receiver by given values
	* @param {float} dz length of translation in x direction*/
	this.translateZ = function(dz) {
		this.z+=dz; this.up();
	}

	/** Update receiver (change positions of stored copies so as stored shifts between receiver and its copies remain constant*/
	this.up = function() {
		for (var i=0; i<this.copies.length; i++) { this.copies[i].setXZ(this.x+this.copiesShifts[i][0],this.z+this.copiesShifts[i][1]); this.copies[i].up(); }
	}
}
/** Creates new Point2d object, for parameters meaning see {@link Point2d}
* @returns {Point2d} new Point2d object*/
Point2d.create = function(x,z) {
	return new Point2d(x,z);
}




/** 3d point implementation
* @class represents points in 3d space
* @param {float} xr x coordinate of point in real space
* @param {float} yr y coordinate of point in real space
* @param {float} zr z coordinate of point in real space
* @property {float} xr x coordinate of point in real space
* @property {float} yr y coordinate of point in real space
* @property {float} zr z coordinate of point in real space
* @property {float} x x coordinate of point in screen projection
* @property {float} y y coordinate of point in screen projection (can be used for visibility issues)
* @property {float} z z coordinate of point in screen projection
* @property {[3x(3 or 4) floats]} p projection matrix
* @property {[Points]} copies array of copies of itself
* @property {[[float,float]]} copiesShifts horizontal and verical shifts of copies from receiver*/
Point3d = function(xr,yr,zr,projection) {
	this.xr = xr || 0;
	this.yr = yr || 0;
	this.zr = zr || 0;
	this.copies = [];
	this.copiesShifts = []
	this.p = projection==undefined? [[1,0,0],[0,1,0],[0,0,1]] : projection;
	this.x = this.p[0][0]*this.xr + this.p[0][1]*this.yr + this.p[0][2]*this.zr + (this.p[0][3] || 0);
	this.y = this.p[1][0]*this.xr + this.p[1][1]*this.yr + this.p[1][2]*this.zr + (this.p[1][3] || 0);
	this.z = this.p[2][0]*this.xr + this.p[2][1]*this.yr + this.p[2][2]*this.zr + (this.p[2][3] || 0);

	/** String representation
	* @returns {String} string representation*/
	this.toString = function() {
		return "Point3d ( "+this.xr+","+this.yr+","+this.zr+" )";
	}

	/** Copy receiver to given positions
	* @param {[[float,float,float]]} shifts vertical and horizontal shifts of copied points
	* @example n = Point3d(100,150,250);
	* shifts=[[100,200,300],[200,200,500],[100,300,100],[200,300,340]]*/
	this.copy = function(shifts) {
		for (var i=0; i<shifts.length; i++) {
			this.copiesShifts.push(shifts[i]);
			this.copies.push(new Point3d(this.xr+shifts[i][0],this.yr+shifts[i][1],this.zr+shifts[i][2]));
		}
	}

	/** Sets new position to receiver
	* @param {float} xr xr coordinate of new position (in real space)
	* @param {float} yr yr coordinate of new position (in real space)
	* @param {float} zr zr coordinate of new position (in real space)*/
	this.setXYZ = function(xr,yr,zr) {
		this.xr=xr; this.yr=yr; this.zr=zr; this.up();
	}

	/** Set new x coordinate to receiver
	* @param x x coordinate of new position (in real space)*/
	this.setX = function(xr) {
		this.xr=xr; this.up();
	}

	/** Set new y coordinate to receiver
	* @param y y coordinate of new position (in real space)*/
	this.setY = function(yr) {
		this.yr=yr; this.up();
	}

	/** Set new z coordinate to receiver
	* @param z z coordinate of new position (in real space)*/
	this.setZ = function(zr) {
		this.zr=zr; this.up();
	}

	/** Change position of receiver by given values
	* @param {float} dx length of translation in x direction (in real space)
	* @param {float} dy length of translation in y direction (in real space)
	* @param {float} dz length of translation in z direction (in real space)*/
	this.translate = function(dx,dy,dz) {
		this.xr+=dx; this.zr+=dz; this.yr+=dy; this.up();
	}

	/** Change horizontal position of receiver by given values
	* @param {float} dx length of translation in x direction (in real space)*/
	this.translateX = function(dx) {
		this.xr+=dx; this.up();
	}

	/** Change horizontal position of receiver by given values
	* @param {float} dy length of translation in y direction (in real space)*/
	this.translateY = function(dy) {
		this.yr+=dy; this.up();
	}

	/** Change vertical position of receiver by given values
	* @param {float} dz length of translation in x direction (in real space)*/
	this.translateZ = function(dz) {
		this.zr+=dz; this.up();
	}

	/** Sets projection matrix
	* @param {[2x(3 or 4) floats]} p new projection matrix*/
	this.setProjection = function(p) { this.p = p; this.up() }

	/** project receiver to xz screen coordinates accordind to this.p*/
	this.project = function() {
		this.x = this.p[0][0]*this.xr + this.p[0][1]*this.yr + this.p[0][2]*this.zr + (this.p[0][3] || 0);
		this.y = this.p[1][0]*this.xr + this.p[1][1]*this.yr + this.p[1][2]*this.zr + (this.p[1][3] || 0);
		this.z = this.p[2][0]*this.xr + this.p[2][1]*this.yr + this.p[2][2]*this.zr + (this.p[2][3] || 0);
	}

	this.rotX = function(a) {
		c = Math.cos(a);
		s = Math.sin(a);
		var p = this.p;
		var p10 = p[1][0], p11 = p[1][1], p12 = p[1][2];
		var p20 = p[2][0], p21 = p[2][1], p22 = p[2][2];
		p[1][0] = p10*c - p20*s;
		p[2][0] = p10*s + p20*c;
		p[1][1] = p11*c - p21*s;
		p[2][1] = p11*s + p21*c;
		p[1][2] = p12*c - p22*s;
		p[2][2] = p12*s + p22*c;
		this.up();
	}

	this.rotY = function(a) {
		c = Math.cos(a);
		s = Math.sin(a);
		var p = this.p;
		var p00 = p[0][0], p01 = p[0][1], p02 = p[0][2];
		var p20 = p[2][0], p21 = p[2][1], p22 = p[2][2];
		var p = this.p;
		p[0][0] = p00*c - p20*s;
		p[2][0] = p00*s + p20*c;
		p[0][1] = p01*c - p21*s;
		p[2][1] = p01*s + p21*c;
		p[0][2] = p02*c - p22*s;
		p[2][2] = p02*s + p22*c;
		this.up();
	}

	this.rotZ = function(a) {
		c = Math.cos(a);
		s = Math.sin(a);
		var p = this.p;
		var p00 = p[0][0], p01 = p[0][1], p02 = p[0][2];
		var p10 = p[1][0], p11 = p[1][1], p12 = p[1][2];
		p[0][0] = p00*c - p10*s;
		p[1][0] = p00*s + p10*c;
		p[0][1] = p01*c - p11*s;
		p[1][1] = p01*s + p11*c;
		p[0][2] = p02*c - p12*s;
		p[1][2] = p02*s + p12*c;
		this.up();
	}

	/** Update receiver (change positions of stored copies so as stored shifts between receiver and its copies remain constant*/
	this.up = function() {
		this.project();
		for (var i=0; i<this.copies.length; i++) { this.copies[i].setXYZ(this.xr+this.copiesShifts[i][0],this.yr+this.copiesShifts[i][1],this.zr+this.copiesShifts[i][2]); this.copies[i].up(); }
	}

	this.up();
}
/** Creates new Point3d object, for parameters meaning see {@link Point3d}
* @returns {Point3d} new Point3d object*/
Point3d.create = function(x,y,z,projection) {
	return new Point3d(x,y,z,projection);
}



/** Line given by ax+bz+c=0 clipped in bounding rectangle
* @class represents line given by equation clipped in bounding rectangle
* @param {Raphael} raphael raphael instance
* @param {float} a a in ax+bz+c=0
* @param {float} b b in ax+bz+c=0
* @param {float} c c in ax+bz+c=0
* @param {Point} lt left top corner of bounding rectangle
* @param {Point} rb right bottom corner of bounding rectangle (lt.x<rb.x && tl.z<rb.z)
* @property {Raphael} raphael Raphael instance
* @property {float} a a in ax+bz+c=0
* @property {float} b b in ax+bz+c=0
* @property {float} c c in ax+bz+c=0
* @property {Point} lt left top corner of bounding rectangle
* @property {Point} rb right bottom corner of bounding rectangle (lt.x<rb.x && tl.z<rb.z)
* @property {Point} point1 1st point of intersection of line with bounding rectangle
* @property {Point} point2 2nd point of intersection of line with bounding rectangle
* @property {Line} line Line object (see {@link Line})*/
LineBR = function(raphael,a,b,c,lt,rb) {
	this.raphael = raphael;
	this.a = a==undefined? 1. : a;
	this.b = b==undefined? 1. : b;
	this.c = c==undefined? 1. : c;
	this.lt = lt || new Point2d();
	this.rb = rb || new Point2d();
	this.point1 = new Point2d();
	this.point2 = new Point2d();
	this.line = new Line(this.raphael,this.point1,this.point2);

	/** Sets new a of ax+bz+c=0
	*@param {float} a new a in ax+bz+c=0*/
	this.setA = function(a) { this.a = a; this.up(); }

	/** Sets new a of ax+bz+c=0
	*@param {float} a new a in ax+bz+c=0*/
	this.setB = function(b) { this.b = b; this.up(); }

	/** Sets new a of ax+bz+c=0
	*@param {float} a new a in ax+bz+c=0*/
	this.setC = function(c) { this.c = c; this.up(); }

	/** Sets new a of ax+bz+c=0
	*@param {float} a new a in ax+bz+c=0*/
	this.setABC = function(a,b,c) { this.a = a; this.b = b; this.c = c; this.up(); }

	/** Hide the line*/
	this.hide = function() { this.point1.setXZ(-100,-100); this.point2.setXZ(-100,-100); }

	/** Finds sections of given line with given bounding rectangle and sets to this.points*/
	this.findSections = function() {
		var x1,z1,x2,z2;
		var a = this.a; var b = this.b; var c = this.c;
		var p1 = this.point1; var p2 = this.point2;
		var lt = this.lt; var rb = this.rb;
		var zmin = lt.z; var zmax = rb.z;
		var xmin = lt.x; var xmax = rb.x;
		if (a == 0.) {
			if (-c/b > zmin && -c/b < zmax) {
				p1.setXZ(xmin,-c/b);
				p2.setXZ(xmax,-c/b);
			}
			else { this.hide(); }
			return;
		}
		if (b == 0.) {
			if (-c/a > xmin && -c/a < xmax) {
				p1.setXZ(-c/a,zmin);
				p2.setXZ(-c/a,zmax);
			}
			else { this.hide(); }
			return;
		}
		x1 = xmin;
		x2 = xmax;
		z1 = -(a*x1+c)/b;
		z2 = -(a*x2+c)/b;
		if (z1<zmin && z2<zmin) { this.hide(); return; }
		if (z1>zmax && z2>zmax) { this.hide(); return; }
		if ((z1>zmin && z1<zmax) && (z2>zmin && z2<zmax)) { p1.setXZ(x1,z1); p2.setXZ(x2,z2); return; }
		if ((z1<zmin && z2>zmax) || (z1>zmax && z2<zmin)) {
			p1.setXZ(-(b*zmin+c)/a,zmin);
			p2.setXZ(-(b*zmax+c)/a,zmax);
			return;
		}
		if (z1 < zmax && z1 > zmin) {
			p1.setXZ(x1,z1);
			if (z2 < zmin) { p2.setXZ(-(b*zmin+c)/a,zmin); }
			else { p2.setXZ(-(b*zmax+c)/a,zmax); }
		}
		else {
			p2.setXZ(x2,z2);
			if (z1 < zmin) { p1.setXZ(-(b*zmin+c)/a,zmin); }
			else { p1.setXZ(-(b*zmax+c)/a,zmax); }
		}
	}

	/** Update receiver*/
	this.up = function() { this.findSections(); this.line.up(); }

	this.up();
}
/** Creates new LineBR object, for parameters meaning see {@link LineBR}
* @returns {LineBR} new LineBR object*/
LineBR.create = function(raphael,a,b,c,lt,rb) {
	return new LineBR(raphael,a,b,c,lt,rb);
}





/** Line
* @class represents line connecting two points
* @param {Raphael} raphael raphael instance
* @param {Point} point1 1st Point2d
* @param {Point} point2 2nd Point2d
* @property {Raphael} raphael Raphael instance
* @property {Point} point1 first point of line
* @property {Point} point2 second point of line
* @property {[Lines]} copies array of copies of receiver
* @property {[[Points],[Points]]} copiesPoints array of points of copies of receiver
* @property {Raphael.el} obj pointer to actual raphael object*/
Line = function(raphael,point1,point2) {
	this.raphael = raphael;
	this.point1 = point1;
	this.point2 = point2;
	this.copies = [];
	this.copiesPoints = [[],[]];
	this.obj = null;

	/** Sets svg path of receiver*/
	this.setPathList = function() { this.pathList = [["M",this.point1.x,this.point1.z],["L",this.point2.x,this.point2.z]]; }
	// Create a raphael object and assign it to this.obj
	this.makeObj = function() { this.obj = this.raphael.path(this.pathList); }
	this.setPathList();
	this.makeObj();
	// Set this.pathList to this.obj raphael object
	this.attrObjPath = function() { this.obj.attr({"path":this.pathList}); }
	/** Update receiver (set this.pathList for given internal variables - x,z,angle... - as well as its copies)*/
	this.up = function() {
		this.setPathList(); this.attrObjPath();
		for (var c=0; c<this.copies.length; c++) { this.copies[c].up(); }
	}
	/** Copy receiver
	* @param {[[Points],[Points]]} points points of to be copied lines*/
	this.copy = function(points) { // points=[[point1,point2,point3],[point4,point5,point6]] wll make lines 14,25,36
		for (var n=0; n<points[0].length; n++) {
			this.copiesPoints[0].push(points[0][n]);
			this.copiesPoints[1].push(points[1][n]);
			this.copies.push(this.makeCopy(points[0][n],points[1][n]));
		}
		this.up();
	}
	/** Returns copy of receiver
	* @returns {Line} copy of receiver*/
	this.makeCopy = function(point1,point2) {
		var ret = new Line(this.raphael,point1,point2);
		ret.obj.remove();
		ret.obj = this.obj.clone();
		return ret;
	}
}
/** Creates new Line object, for parameters meaning see {@link Line}
* @returns {Line} new Line object*/
Line.create = function(raphael,point1,point2) {
	return new Line(raphael,point1,point2);
}



/** Path (for easier modifying)
* @class represents svg path
* @param {Raphael} raphael raphael instance
* @param {Array} pathList path list for raphael path
* @property {Raphael} raphael raphael instance
* @property {Array} pathList svg array (list) of the object
* @property {Raphael.el} obj actual raphael object*/
Path = function(raphael,pathList) {
	this.pathList = pathList==undefined? [] : pathList;
	this.raphael = raphael;
	this.obj = this.raphael.path(this.pathList);
	// methods
	// Set this.pathList to this.obj raphael object
	this.attrObjPath = function() { this.obj.attr({"path":this.pathList}); }

	/** Sets one specific value in this.pathList (this.pathList[i][j] = val)
	* @param {int} i first index
	* @param {int} j second index
	* @param {float|string} val value to be set*/
	this.setItem = function(i,j,val) { this.pathList[i][j] = val; this.attrObjPath(); }

	/** Increment one specific value in this.pathList (this.pathList[i][j] = val)
	* @param {int} i first index
	* @param {int} j second index
	* @param {float} val value of incerementation*/
	this.iaddItem = function(i,j,val) { this.pathList[i][j] += val; this.attrObjPath(); }

	/** Sets new svg line path to receiver
	* @param {Array} newPathList newpath list*/
	this.setNewPath = function(newPathList) { this.pathList = newPathList; this.attrObjPath(); }
}
/** Creates new Path object, for parameters meaning see {@link Path}
* @returns {Path} new Path object*/
Path.create = function(pathList) {
	return new Path(pathList);
}



/** Circle (for easier modifying)
* @class represents circle
* @param {Raphael} raphael raphael instance
* @param {float} [cx=0.] x coordinate of circle center
* @param {float} [cz=0.] z coordinate of circle center
* @param {float} [r=10.] circle radius
* @property {Raphael} raphael raphael instance
* @property {float} x x coordinate of circle center
* @property {float} z z coordinate of circle center
* @property {float} r radius of the circle
* @property {Raphael.circle} obj actual raphael object
* @property {[Circles]} copies array of copies of receiver
* @property {[[float,float]]} copiesShifts array of shifts of copies of receiver*/
Circle = function(raphael,x,z,r) {
	this.raphael = raphael;
	this.x = x==undefined? 0. : x;
	this.z = z==undefined? 0. : z;
	this.r = r==undefined? 0. : r;
	this.copies = [];
	this.copiesShifts = [];
	this.obj = this.raphael.circle(this.x,this.z,this.r);

	/** Sets radius of receiver
	* @param {float} r new radius*/
	this.setR = function(r) { this.r = r; this.obj.attr({"r":r}); this.up(); }

	/** Sets x coordinate of receiver
	* @param {float} x new x coordinate*/
	this.setX = function(x) { this.x = x; this.obj.attr({"cx":x}); this.up(); }

	/** Sets z coordinate of receiver
	* @param {float} z new z coordinate*/
	this.setZ = function(z) { this.z = z; this.obj.attr({"cy":z}); this.up(); }

	/** Sets new coordinates of receiver
	* @param {float} x new x coordinate
	* @param {float} z new z coordinate*/
	this.setXZ = function(x,z) { this.x = x; this.z = z; this.obj.attr({"cx":x,"cy":z}); }

	/** Translate along x axis of receiver
	* @param {float} dx x distance of translation*/
	this.translateX = function(dx) { this.x += dx; this.obj.attr({"cx":this.x}); }

	/** Translate along z axis of receiver
	* @param {float} dz z distance of translation*/
	this.translateZ = function(dz) { this.z += dz; this.obj.attr({"cy":this.z}); }

	/** Translate receiver
	* @param {float} dx x distance of translation
	* @param {float} dz z distance of translation*/
	this.translate = function(dx,dz) { this.x += dx; this.z += dz; this.obj.attr({"cx":this.x,"cy":this.z}); }

	/** Copy of receiver
	* @param {[[float,float]]} shifts shifts of copies*/
	this.copy = function(shifts) { for (var i=0; i<shifts.length; i++) { this.copiesShifts.push(shifts[i]); this.copies.push(this.makeCopy()); this.up()} }
	this.makeCopy = function() {
		var ret = new Circle(this.raphael,this.x,this.z,this.r)
		ret.obj.remove();
		ret.obj = this.obj.clone();
		return ret;
	}

	/** Update*/
	this.up = function() { 
		for (var c=0; c<this.copies.length; c++) {
			this.copies[c].setXZ(this.x+this.copiesShifts[c][0],this.z+this.copiesShifts[c][1]);
			this.copies[c].setR(this.r);
		}
	}
}
/** Creates new Circle object, for parameters meaning see {@link Circle}
* @returns {Circle} new Circle object*/
Circle.create = function(raphael,x,z,r) {
	return new Circle(raphael,x,z,r);
}




/** Ellipse
* @class represents ellipse
* @param {Raphael} raphael raphael instance
* @param {float} [cx=0.] x coordinate of circle center
* @param {float} [cz=0.] z coordinate of circle center
* @param {float} [a=10.] x semiaxis
* @param {float} [b=10.] z semiaxis
* @param {float} [angle=0.] rotation
* @param {int} [nSeg=0.] number of segments
* @property {Raphael} raphael raphael instance
* @property {float} cx x coordinate of circle center
* @property {float} cz z coordinate of circle center
* @property {float} a x semiaxis
* @property {float} b z semiaxis
* @property {float} angle rotation
* @property {int} nSeg number of segments*/
Ellipse = function(raphael,cx,cz,a,b,angle,nSeg) {
	this.raphael = raphael;
	this.cx = cx==undefined? 0. : cx;
	this.cz = cz==undefined? 0. : cz;
	this.a = a==undefined? 10. : a;
	this.b = b==undefined? 10. : b;
	this.angle = angle==undefined? 0. : angle;
	this.nSeg = nSeg==undefined? 32 : nSeg;
	this.pathList = [];
	this.obj = this.raphael.path([]);

	/** Sets a (x semiaxis) of receiver
	* @param {float} r new radius*/
	this.setA = function(a) { this.a = a; this.up(); }

	/** Sets b (z semiaxis) of receiver
	* @param {float} r new radius*/
	this.setB = function(b) { this.b = b; this.up(); }

	/** Sets a (x semiaxis) and b (z semiaxis) of receiver
	* @param {float} a new a (x semiaxis)
	* @param {float} b new b (x semiaxis)*/
	this.setAB = function(a,b) { this.a = a; this.b = b; this.up(); }

	/** Sets x coordinate of center of receiver
	* @param {float} cx new x coordinate*/
	this.setX = function(cx) { this.cx = cx; this.up(); }

	/** Sets z coordinate of center of receiver
	* @param {float} cz new z coordinate*/
	this.setZ = function(cz) { this.cz = cz; this.up(); }

	/** Sets new coordinates of receiver
	* @param {float} cx new x coordinate
	* @param {float} cz new z coordinate*/
	this.setXZ = function(cx,cz) { this.cx = cx; this.cz = cz; this.up(); }

	/** Sets new coordinates of receiver and new semiaxes
	* @param {float} cx new x coordinate
	* @param {float} cz new z coordinate
	* @param {float} a new a (x semiaxis)
	* @param {float} b new b (x semiaxis)
	* @param {float} angle new angle*/
	this.setAll = function(cx,cz,a,b,angle) { this.cx = cx; this.cz = cz; this.a = a; this.b = b; this.angle = angle; this.up(); }

	/** Sets rotation of receiver
	* @param {float} cz new z coordinate*/
	this.setAngle = function(angle) { this.angle = angle; this.up(); }

	/** Translate along x axis of receiver
	* @param {float} dx x distance of translation*/
	this.translateX = function(dx) { this.cx += dx; this.up(); }

	/** Translate along z axis of receiver
	* @param {float} dz z distance of translation*/
	this.translateZ = function(dz) { this.cz += dz; this.up(); }

	/** Translate receiver
	* @param {float} dx x distance of translation
	* @param {float} dz z distance of translation*/
	this.translate = function(dx,dz) { this.cx += dx; this.cz += dz; this.up(); }

	/** Sets svg path of receiver*/
	this.setPathList = function() {
		this.pathList = [];
		var c = Math.cos(this.angle);
		var s = Math.sin(this.angle);
		var t,xl,zl;
		for (var i=0; i<this.nSeg; i++) {
			t = i*2*Math.PI/this.nSeg;
			xl = this.a*Math.cos(t);
			zl = this.b*Math.sin(t);
			if (i==0) { this.pathList[0] = ["M",this.cx+c*xl+s*zl,this.cz-s*xl+c*zl]; }
			else { this.pathList[i] = ["L",this.cx+c*xl+s*zl,this.cz-s*xl+c*zl]; }
		}
		this.pathList.push(["Z"]);
		this.obj.attr({"path":this.pathList});
	}

	/** Update*/
	this.up = function() { 
		this.setPathList();
	}

	this.up();
}
/** Creates new Ellipse object, for parameters meaning see {@link Ellipse}
* @returns {Circle} new Ellipse object*/
Ellipse.create = function(raphael,cx,cz,a,b,angle,nSeg) {
	return new Ellipse(raphael,cx,cz,a,b,angle,nSeg);
}




/** Slider
* @class abstract class epresenting generic slider
* @param {Raphael} raphael raphael instance
* @param {Object} params parameters,see below
* @param {float} [params.x=100.] x coordinate of left/upper end
* @param {float} [params.z=100.] z coordinate of left/upper end
* @param {float} [params.length=100.] length of slider [pixels]
* @param {float} [params.rcirc=5.] radius of slider's circle
* @param {float} [params.val1=0.] value when the slider is at the very beginning
* @param {float} [params.val2=1.] value when the slider is at the very end
* @param {float} [params.initVal=(val1+val2)/2] initial value
* @param {float} [params.gap=3.] gap between end of the slider's line and the slider's circle when the circle is at extreme position
* @property {Raphael} raphael raphael instance
* @property {float} x x coordinate of left/upper end
* @property {float} z z coordinate of left/upper end
* @property {float} len len of slider [pixels]
* @property {float} rcirc radius of slider's circle
* @property {float} val1 value when the slider is at the very beginning
* @property {float} val2 value when the slider is at the very end
* @property {float} initVal initial value
* @property {float} gap gap between end of the slider's line and the slider's circle when the circle is at extreme position
* @property {float} lenVals length span of getable values
* @property {Raphael.el} line line
* @property {Circle} circ Circle*/
Slider = function(raphael,params) {
	this.constructorSlider = function(raphael,params) {
		this.raphael = raphael;
		this.x = params.x==undefined? 100. : params.x;
		this.z = params.z==undefined? 100. : params.z;
		this.len = params.length==undefined? 100. : params.length;
		this.rcirc = params.rcirc==undefined? 5. : params.rcirc;
		this.gap = params.gap==undefined? 3. : params.gap;
		this.start = this.gap+this.rcirc;
		this.stop = this.len - this.start;
		this.val1 = params.val1==undefined? 0. : params.val1;
		this.val2 = params.val2==undefined? 1. : params.val2;
		this.initVal = params.initVal==undefined? .5*(this.val1+this.val2) : params.initVal;
		this.lenVals = this.len - 2*this.start;
		this.line = this.raphael.rect(this.x,this.z-2,this.len,4).attr({"fill":"#bbb"});
		this.circ = new Circle(this.raphael,this.x,this.z,this.rcirc);
		this.circ.obj.attr({"cursor":"move","fill":"#f00"})
		this.circ.obj.slider = this;
		this.circ.obj.circ = this.circ;
	}
}


/** Horizontal slider
* @class represents horizontal slider
* @param {Raphael} raphael raphael instance
* @param {Object} params parameters, see below
* @param {float} [params.x=100.] x coordinate of left/upper end
* @param {float} [params.z=100.] z coordinate of left/upper end
* @param {float} [params.length=100.] length of slider [pixels]
* @param {float} [params.rcirc=5.] radius of slider's circle
* @param {float} [params.val1=0.] value when the slider is at the very beginning
* @param {float} [params.val2=1.] value when the slider is at the very end
* @param {float} [params.initVal=(val1+val2)/2] initial value
* @param {float} [params.gap=3.] gap between end of the slider's line and the slider's circle when the circle is at extreme position
* @property {Raphael} raphael raphael instance
* @property {float} x x coordinate of left/upper end
* @property {float} z z coordinate of left/upper end
* @property {float} len len of slider [pixels]
* @property {float} rcirc radius of slider's circle
* @property {float} val1 value when the slider is at the very beginning
* @property {float} val2 value when the slider is at the very end
* @property {float} initVal initial value
* @property {float} gap gap between end of the slider's line and the slider's circle when the circle is at extreme position
* @property {float} lenVals length span of getable values
* @property {Raphael.el} line line
* @property {Circle} circ circle*/
HorizontalSlider = function(raphael,params) {
	this.inheritFrom = Slider; this.inheritFrom();

	/** Constructor, see {@link HorizontalSlider} for parameters description*/
	this.constructorHorizontalSlider = function(raphael,params) {
		this.constructorSlider(raphael,params);
		var initPos;
		if (this.lenVals<0) {
			initPos = this.len*.5;
			this.lenVals = 1.;
		}
		else {
			initPos = this.start+this.lenVals/(this.val2-this.val1)*(this.initVal-this.val1);
			if (initPos < this.start) { initPos = this.start; }
			if (initPos > this.stop)  { initPos = this.stop; }
		}
		this.circ.setXZ(this.x+initPos,this.z);
		this.circ.obj.dragReset = function() { this.slider.pos = this.circ.x; };
		this.circ.obj.dragReset();
		this.circ.obj.dragUpdate = function(dx,dz) {
			this.slider.pos += dx;
			if (this.slider.pos >= this.slider.x+this.slider.start && this.slider.pos <= this.slider.x+this.slider.len-this.slider.start) {
				this.circ.setX(this.slider.pos);
				this.slider.onmove();
			}
		}
		/**@ignore*/
		this.circ.obj.drag(dragMove,dragInit,dragReset);
	}

	/** Get current value from receiver
	* @returns {float} current value*/
	this.getVal = function() {
		var dx = this.circ.x-this.x-this.start;
		return this.val1+dx*(this.val2-this.val1)/this.lenVals;
	}

	/** Set value to receiver
	* @param {float} new value*/
	this.setVal = function(newVal) {
		var newPos;
		if (this.lenVals<0) {
			newPos = this.len*.5;
			this.lenVals = 1.;
		}
		else {
			newPos = this.start+this.lenVals/(this.val2-this.val1)*(newVal-this.val1);
			if (newPos < this.start) { newPos = this.start; }
			if (newPos > this.stop)  { newPos = this.stop; }
		}
		this.circ.setXZ(this.x+newPos,this.z);
		this.circ.obj.dragReset();
	}

	/** What to do when moving. Initially empty function, overwrite it by desired actions*/
	this.onmove = function() {}

	this.constructorHorizontalSlider(raphael,params);
}
/** Creates new HorizontalSlider object, for parameters meaning see {@link HorizontalSlider}
* @returns {HorizontalSlider} new HorizontalSlider object*/
HorizontalSlider.create = function(raphael,params) {
	return new HorizontalSlider(raphael,params);
}



/** Button
* @class represents button
* @param {Raphael} raphael raphael instance
* @param {Object} params parameters, see below
* @param {string} [params.str="str"] displayed string of button
* @param {float} [params.x=100.] x coordinate of left upper end
* @param {float} [params.z=100.] z coordinate of left upper end
* @param {float} [params.width=100.] length of slider [pixels]
* @param {float} [params.height=30.] length of slider [pixels]
* @param {float} [params.fontSize=20.] font size of button str
* @param {float} [params.rCorner=4.] radius of button corners camber
* @param {color} [params.bgColor="#bbb"] background color of button
* @param {color} [params.bgColorWhenClicked="#aaa"] background color of button when clicked
* @property {Raphael} raphael raphael instance
* @property {float} x x coordinate of left upper end
* @property {float} z z coordinate of left upper end
* @property {float} width length of slider [pixels]
* @property {float} height length of slider [pixels]
* @property {float} fontSize font size of button text
* @property {color} bgColor background color of button
* @property {color} bgColorWhenClicked background color of button when clicked
* @property {Raphael.rect} rect rectangle of receiver
* @property {Raphael.text} text object of receiver*/
Button = function(raphael,params) {
	this.constructorButton = function(raphael,params) {
		this.raphael = raphael;
		var str = params.str==undefined? 'str' : params.str;
		this.x = params.x==undefined? 100. : params.x;
		this.z = params.z==undefined? 100. : params.z;
		this.width = params.width==undefined? 100. : params.width;
		this.height = params.height==undefined? 30. : params.height;
		var fontSize = params.fontSize==undefined? 20. : params.fontSize;
		this.bgColor = params.bgColor==undefined? "#bbb" : params.bgColor;
		this.bgColorWhenClicked = params.bgColorWhenClicked==undefined? "#aaa" : params.bgColorWhenClicked;
		var rCorner = params.rCorner==undefined? 4. : params.rCorner;
		this.rect = this.raphael.rect(this.x,this.z,this.width,this.height,rCorner).attr({"fill":this.bgColor,"cursor":"pointer"})
		this.rect.button = this;
		this.text = this.raphael.text(this.x+.5*this.width,this.z+.5*this.height,str).attr({"font-size":fontSize,"cursor":"pointer"})
		this.text.button = this;

		/** What to do when clicked. Initially empty function, overwrite it by desired actions*/
		this.onclick = function() {}

		this.doThisWhenClicked = function() {
			this.text.attr({"x":this.x+.5*this.width+2})
			this.rect.attr({"fill":this.bgColorWhenClicked})
			this.onclick();
		}
		this.doThisWhenUnclicked = function() {
			this.text.attr({"x":this.x+.5*this.width})
			this.rect.attr({"fill":this.bgColor})
		}
		this.rect.mousedown(function(event) {this.button.doThisWhenClicked();})
		this.rect.mouseup(function(event) {this.button.doThisWhenUnclicked();})
		this.text.mousedown(function(event) {this.button.doThisWhenClicked();})
		this.text.mouseup(function(event) {this.button.doThisWhenUnclicked();})
		this.rect.mouseout(function(event) {this.button.doThisWhenUnclicked();})
		this.text.mouseout(function(event) {this.button.doThisWhenUnclicked();})
	}

	this.constructorButton(raphael,params);
}
/** Creates new Button object, for parameters meaning see {@link Button}
* @returns {Button} new Button object*/
Button.create = function(raphael,params) {
	return new Button(raphael,params);
}




/** Text in rectangular frame
* @class represents text in rectangular frame
* @param {Raphael} raphael rapheal instance
* @param {Object} params parameters, see below
* @param {float} [params.x=100.] x coordinate of left lower corner [pixel]
* @param {float} [params.z=100.] z coordinate of left lower corner [pixel]
* @param {float} [params.width=100.] width [pixel]
* @param {float} [params.height=100.] height [pixel]
* @param {string} [params.str="string"] string to be shown
* @param {float} [params.fontSize=20.] font size
* @property {float} x x coordinate of top left corner
* @property {float} z z coordinate of top left corner
* @property {float} width width of rectangle
* @property {float} height height of rectangle
* @property {string} str text to display
* @property {float} fontSize font size
* @property {Raphael.text} text raphael text object
* @property {Raphael.rect} rect raphael rect object*/
TextRect = function(raphael,params) {

	/** Constructor, see {@link TextRect} for parameters description*/
	this.constructorTextRect = function(raphael,params) {
		this.x = params.x==undefined? 100. : params.x;
		this.z = params.z==undefined? 100. : params.z;
		this.width = params.width==undefined? 100. : params.width;
		this.height = params.height==undefined? 100. : params.height;
		this.str = params.str==undefined? "string" : params.str;
		this.fontSize = params.fontSize==undefined? 20 : params.fontSize;
		this.rect = raphael.rect(this.x,this.z-this.height,this.width,this.height);
		this.text = raphael.text(this.x+.5*this.width,this.z-.5*this.height,this.str).attr({"font-size":this.fontSize});
	}

	/** Update receiver (set this.pathList for given internal variables)*/
	this.up = function() {
		this.rect.attr({"x":this.x,"y":(this.z-this.height)});
		this.text.attr({"x":(this.x+.5*this.width),"y":(this.z-.5*this.height),"text":this.str,"font-size":this.fontSize});
	}

	/** Sets new x position of receiver (this.x = x)
	* @param {float} x new x coordinate [pixel]*/
	this.setX = function(x) { this.x = x; this.up(); }

	/** Sets new z position of receiver (this.z = z)
	* @param {float} z new z coordinate [pixel]*/
	this.setZ = function(z) { this.z = z; this.up(); }

	/** Sets new x and z position of receiver (this.x = x; this.z = z)
	* @param {float} x new x coordinate [pixel]
	* @param {float} z new z coordinate [pixel]*/
	this.setXZ = function(x,z) { this.x = x; this.z = z; this.up(); }

	/** Sets new text of receiver
	* @param {string} str new string*/
	this.setStr = function(str) { this.str = str; this.up(); }

	this.constructorTextRect(raphael,params);
}
/** Creates new TextRect object, for parameters meaning see {@link TextRect}
* @returns {TextRect} new TextRect object*/
TextRect.create = function(raphael,params) {
	return new TextRect(raphael,params);
}





